# from ...hub.hub_options import HubOptions
# from .chargecontroller import ChargeController
# from .chargecontroller_lite import ChargeControllerLite

# class ChargeControllerFactory:
#     @staticmethod
#     def create(options: HubOptions):
#         if options.peaqev_lite:
#             return ChargeControllerLite(options)
#         return ChargeController(options)